/*
 * ServoController.c
 *
 * Created: 2015-01-01 19:26:18
 *  Author: Niejeden
 */ 

#define STEROWANIE 1 // Sposb sterowania: 0 - impuls, 1 - podanie masy

#define SERVO_MIN 1000 // 600
#define SERVO_MAX 2300 //2648

#include <avr/io.h>
#include <avr/interrupt.h>

const uint8_t przelicznik_adc = (SERVO_MAX - SERVO_MIN)/1024;

volatile uint16_t potencjometr[5], // tablica przechowujca stan potencjometrw
				  wypelnienie_aktualne[2], // aktualny PWM podawany na serwo
				  wypelnienie_zadane[2]; // PWM docelowy

volatile struct _flagi {
	uint8_t blokada_przycisku_1:1,
	blokada_przycisku_2:1,
	pozycja_serwa_1:1,
	pozycja_serwa_2:1;
} flagi;

ISR(TIM0_OVF_vect)
{
		
	if (flagi.pozycja_serwa_1) // w zalenoci od zadanej pozycji serwa
	{
		wypelnienie_zadane[0] = (SERVO_MIN + (potencjometr[1] * przelicznik_adc)); // oblicz zadany docelowy PWM
		if (wypelnienie_aktualne[0] == wypelnienie_zadane[0]) // jeli osignito zadane wypenienie (tzn. ruch dobieg koca)
		{
			PORTB |= (1<<1); // przecz przekanik
		}
	}
	else
	{
		wypelnienie_zadane[0] = (SERVO_MIN + (potencjometr[0] * przelicznik_adc)); // oblicz zadany docelowy PWM
		if (wypelnienie_aktualne[0] == wypelnienie_zadane[0]) // jeli osignito zadane wypenienie (tzn. ruch dobieg koca)
		{
			PORTB &= ~(1<<1); // przecz przekanik
		}		
	}
	
	if (flagi.pozycja_serwa_2)
	{
		wypelnienie_zadane[1] = (SERVO_MIN + (potencjometr[3] * przelicznik_adc));
		if (wypelnienie_aktualne[1] == wypelnienie_zadane[1])
		{
			PORTB |= (1<<0);
		}
	}
	else
	{
		wypelnienie_zadane[1] = (SERVO_MIN + (potencjometr[2] * przelicznik_adc));
		if (wypelnienie_aktualne[1] == wypelnienie_zadane[1])
		{
			PORTB &= ~(1<<0);
		}
	}
		
	static uint16_t cykl_opoznienia = 0; // Spowolnienie ruchu
	if (cykl_opoznienia++ >= (potencjometr[4] / 10))
	{
		cykl_opoznienia = 0;
		if (wypelnienie_aktualne[0] > wypelnienie_zadane[0])
		{
			--wypelnienie_aktualne[0];
		}
		
		if (wypelnienie_aktualne[0] < wypelnienie_zadane[0])
		{
			++wypelnienie_aktualne[0];
		}
			
	    if (wypelnienie_aktualne[1] > wypelnienie_zadane[1])
		{
		    --wypelnienie_aktualne[1];
		}
				  
		if (wypelnienie_aktualne[1] < wypelnienie_zadane[1])
		{
		    ++wypelnienie_aktualne[1];
		}
	}
		
	OCR1A = wypelnienie_aktualne[0];
	OCR1B = wypelnienie_aktualne[1];
	
	static uint8_t cykl_przyciskow = 0; // debouncing przyciskw
		
    if (cykl_przyciskow++ > 70)
	{
		cykl_przyciskow = 0;
		static uint8_t stan_przyciskow_nowy = 0, stan_przyciskow_stary = 0;
				 	
		stan_przyciskow_nowy = (~PINB & (1<<PB2)) | (~PINA & (1<<PA7)); // odczytaj stan przyciskw
				 	
		if (stan_przyciskow_stary == stan_przyciskow_nowy) // jeli stan przyciskw si nie zmieni od ostatniego odczytu
		{
		 	#if STEROWANIE // jeli wybrano sterowanie mas
			 
			if (stan_przyciskow_nowy & (1<<PA7)) // jeli pin sterowania 1 zwarty do masy
			{
				flagi.pozycja_serwa_1 = 1;
			}
			else // jeli nie zwarty
			{
				flagi.pozycja_serwa_1 = 0;
			}
			
			if (stan_przyciskow_nowy & (1<<PB2)) // jeli pin sterowania 2 zwarty do masy
			{
				flagi.pozycja_serwa_2 = 1;
			}
			else // jeli nie zwarty
			{
				flagi.pozycja_serwa_2 = 0;
			}
			
			#else // jeli wybrano sterowanie impulsem
			
			if (stan_przyciskow_nowy & (1<<PA7)) // jeli wcinity przycisk 1
		 	{
			 	if (!flagi.blokada_przycisku_1) // jeli przycisk nie jest zablokowany
			 	{
				 	flagi.pozycja_serwa_1 ^= 0x01; //zmie pozycj serwa 1 na przeciwn
				 	flagi.blokada_przycisku_1 = 1; // zablokuj przycisk
			 	};
		 	}
		 	else // jeli przycisk puszczony
		 	{
			 	flagi.blokada_przycisku_1 = 0; // zwolnij blokad przycisku
		 	};
					 	
		 	if (stan_przyciskow_nowy & (1<<PB2)) // jeli wcinity przycisk 2
		 	{
			 	if (!flagi.blokada_przycisku_2) // jeli przycisk nie jest zablokowany
			 	{
				 	flagi.pozycja_serwa_2 ^= 0x01; //zmie pozycj serwa 2 na przeciwn
				 	flagi.blokada_przycisku_2 = 1; // zablokuj przycisk
			 	};
		 	}
		 	else // jeli przycisk puszczony
		 	{
			 	flagi.blokada_przycisku_2 = 0; // zwolnij blokad przycisku
		 	}
		
		#endif
		 }
		 
		stan_przyciskow_stary = stan_przyciskow_nowy; // zapamitaj nowy stan
	 }
}

void sprawdz_potencjometry(void)
{
	for (uint8_t i = 0; i<5; ++i) // ptla odczytujca stan potencjometrw
	{
		ADMUX = i; //  wybr kanau przetwornika (potencjometru do odczytania)
		ADCSRA |= (1<<ADSC); // uruchom pojedynczy pomiar - wynik do odrzucenia ze wzgldu na zmian kanau
		while(ADCSRA & (1<<ADSC)); // odczekaj do koca pomiaru
		ADCSRA |= (1<<ADSC); // uruchom pojedynczy pomiar
		while(ADCSRA & (1<<ADSC)); // odczekaj do koca pomiaru
		potencjometr[i] = ADCW; // zapisz odczytan warto
	}
}

int main(void)
{
  TCCR0B = (1<<CS00); // uruchom Timer 0
  TIMSK0 = (1<<TOIE0); // uruchom przerwania Timer 0  
  
  DDRB = 0xFF;
  DDRB &= ~(1<<2);
  PORTB |= (1<<2);
  
  DDRA = 0xFF;
  DDRA &= ~((1<<7) | (1<<4) | (1<<3) | (1<<2) | (1<<1) | (1<<0));
  PORTA |= (1<<7);
  
  ADCSRA =  (1<<ADEN) | (1<<ADPS1) | (1<<ADPS0); // prescaller ADC = 8
  ADMUX = 0x00; // kana ADC 0, Vcc jako Vref
  
  sprawdz_potencjometry(); // pierwszy odczyt potencjometrw, przed wejciem w gwn ptl
  wypelnienie_aktualne[0] = (SERVO_MIN + (potencjometr[0] * przelicznik_adc)); // pierwsze przeliczenie pozycji serwa 0
  wypelnienie_aktualne[1] = (SERVO_MIN + (potencjometr[2] * przelicznik_adc)); // pierwsze przeliczenie pozycji serwa 1
  
  TCCR1A |= 1<<WGM11 | 1<<COM1A1 | 1<<COM1B1;
  TCCR1B |= 1<<WGM13 | 1<<WGM12 | 1<<CS10;
  ICR1 = (F_CPU / 50) -1;
  
  OCR1A = wypelnienie_aktualne[0]; // pierwsze ustawienie pozycji serwa 0 (bez opnienia ruchu)
  OCR1B = wypelnienie_aktualne[1]; // pierwsze ustawienie pozycji serwa 1 (bez opnienia ruchu)

  sei();
   
  while(1)
  {
	 sprawdz_potencjometry();
  }
  
}